/*
 * Project: Spherelib
 * Date: 08/10  
 * Authors: J.M. Colley - M. Le Jeune
 *
 */

/* Copyright (C) 2008 APC CNRS Université Paris Diderot <lejeune@apc.univ-paris7.fr>  
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.html
 */


#include "CD_CommonDefinition.h"
#include "arr.h"
#include <assert.h>
#include "Bin.h"

using namespace std; 

double* Bin::GetNbMode()
{
  double* BinNbMode = new double[m_NBand];
  if (BinNbMode==NULL) {
    CD_WarningStatus(7);
    return NULL;      
  }
  
  long lmin; // minimum l value of bin
  long lmax; // maximum l value of bin

  for (long q=0; q<m_NBand; q++)
    {
      BinNbMode[q] = 0;
      
      // Get the bin limits
      lmin  = getLmin(q);
      lmax  = getLmax(q);
      
      // Bin the stats
      for (long ll=lmin; ll<=lmax; ll++)
	BinNbMode[q]  += (2*ll+1)*(m_SpecBin[q][ll]); 
    }
  
  return (BinNbMode);
}


arr<double>* Bin::UnbinSpec (arr<double>* Spec)
{
  arr<double>* unbinSpec = new arr<double> (m_Lmax+1);
  unbinSpec->fill(0);

  long lmin; // minimum l value of bin
  long lmax; // maximum l value of bin
  
  for (long q=0; q<m_NBand; q++)
    { 
      lmin = getLmin(q);
      lmax = getLmax(q);
      
      for (long ll=lmin; ll<=lmax; ll++)
	(*unbinSpec)[ll] += (*Spec)[q]*((m_SpecBin)[q][ll]);
    }

  return unbinSpec;
}

Status Bin::Rescale (long Lmax)
{
  
  long newLmax  = m_Lmax;
  long newNBand = m_NBand;
  
  if (Lmax < m_Lmax)
    {
      for (long q=0; q<m_NBand; q++) // looking for the bin which contain lmax
	{
	  if ((m_SpecBin)[q][Lmax]!= 0) // found one !
	    {
	      newNBand = q;
	      // is lmax in the growing part of the window ? 
	      if ((m_SpecBin)[newNBand][Lmax] >= (m_SpecBin)[newNBand][Lmax+1])
		newNBand++; // if not , take the following bin
	      
	      // new lmax of window function is the one of this bin
	      newLmax = getLmax(newNBand);
	      break;
	    }
	}
     
      // Build a new array with those new dimensions
      arr2<double> *newBin = new arr2<double>(newNBand, newLmax+1);
      
      // Fill it with the old values
      for (long q=0; q<newNBand; q++)
	for (long ll=0; ll<=newLmax; ll++) 
	  (*newBin)[q][ll] = m_SpecBin[q][ll];
      
      Set(*newBin);

    }
  CD_returnOK;
  
  
}


