/*
 * Project: Spherelib
 * Date: 08/10  
 * Authors: J.M. Colley - M. Le Jeune
 *
 */

/* Copyright (C) 2008 APC CNRS Université Paris Diderot <lejeune@apc.univ-paris7.fr>  
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.html
 */

#ifndef TYPECOMMUN_H
#define TYPECOMMUN_H

typedef long Status;


/*********************************************************
 *                DEFINE 
 ********************************************************/
#define CD_LMAX 300
#define CD_MMAX CD_LMAX

#define printStatusMes(Status){\
    string Mes = "";\
    switch (Status)	       \
      {\
      case 0:Mes="OK";break;\
      case 1:Mes="Empty object";break;\
      case 2:Mes="Lmax/Mmax negative";break;\
      case 3:Mes="Lmax is higher than AlmExt object dimension (the lower one will be used)";break;\
      case 6:Mes="Need SHT weight files. Define PathWeight with SetPathWeight() method";break;\
      case 7:Mes="Insufficient memory";break;\
      case 8:Mes="Npix is higher than MapExt object dimension (the lower one will be used)";break;\
      case 9:Mes="Npix negative";break;\
      case 10:Mes="Dimension are not compatible";break;	\
      case 12:Mes="Number of object is <=0";break;\
      case 13:Mes="File doesn't exist";break;\
      case 15:Mes="Nside < 0";break;\
      case 16:Mes="Nside not a power of 2";break;\
      case 17:Mes="Number of bins doesn't match";break;\
      default:break;\
      }		    \
    fprintf(stderr, "\n%s\n",Mes.c_str());			\
}




/*********************************************************
 *                External Test object and function 
 ********************************************************/

// check status object and return if NOK with warning message
#define CD_RetIfObjNok(Obj) { \
if (Obj.GetStatus() > 0) {\
fprintf(stderr,"\nWARNING !!! Anomalous status object %d [File:'%s' Line=%d]", \
(int)Obj.GetStatus(), __FILE__, __LINE__); printStatusMes(Obj.GetStatus);		   \
return(Obj.GetStatus());		   \
}}


// eval func, check status and return if NOK with warning message
#define CD_RetIfNok(Func) { \
Status Ret = Func; \
if (Ret > 0) {\
fprintf(stderr,"\nWARNING !!! Anomalous status function %d [File:'%s' Line=%d]", \
(int)Ret, __FILE__, __LINE__); printStatusMes(Ret);\
return(Ret); \
}}


// check status object and exit if NOK with warning message
#define CD_ExitIfObjNok(Obj) { \
if (Obj.GetStatus() > 0) {\
fprintf(stderr,"\nWARNING !!! Anomalous status object %d [File:'%s' Line=%d]\n", \
(int)Obj.GetStatus(), __FILE__, __LINE__); printStatusMes(Obj.GetStatus);\
exit((int)Obj.GetStatus()); \
}}


// eval func, check status and exit if NOK with warning message
#define CD_ExitIfNok(Func) { \
Status Ret = Func; \
if (Ret > 0) {\
fprintf(stderr,"\nERROR !!! function failed with status %d [File:'%s' Line=%d]\n", \
(int)Ret, __FILE__, __LINE__); printStatusMes(Ret);	       \
exit((int)Ret); \
}}



/*********************************************************
 *                Internal test status and function 
 ********************************************************/

// return if m_status is NOK and add warning message
#define CD_iCheckStatus \
{if (this->m_Status > 0) {\
fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)this->m_Status, __FILE__, __LINE__); printStatusMes((int)this->m_Status);	  \
return(this->m_Status); \
}}

// return if m_status is NOK and add warning message
#define CD_iExit \
{\
fprintf(stderr,"\nEXIT !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)this->m_Status, __FILE__, __LINE__); printStatusMes((int)this->m_Status);	  \
exit(1); \
}

// return if m_status is NOK and add warning message
#define CD_iExitMess(MyString) \
{\
fprintf(stderr,"\n"); \
fprintf(stderr,MyString); \
fprintf(stderr,"\nEXIT !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)this->m_Status, __FILE__, __LINE__); \
 printStatusMes((int)this->m_Status);	  \
exit(1); \
}

// eval func, update m_Status and return if NOK with warning message
#define CD_iCheck(fnc) {this->m_Status = fnc; \
CD_iCheckStatus;} 

// update m_Status, add warning message if NOK status and return
#define CD_return(status) {this->m_Status = status; \
if (this->m_Status > 0) {\
fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)status, __FILE__, __LINE__);printStatusMes((int)status); }\
return(this->m_Status);} 

// update m_Status, add warning message if NOK status and return
#define CD_returnMes(status,MyMes) {this->m_Status = status; \
if (this->m_Status > 0) {\
fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)status, __FILE__, __LINE__);printStatusMes((int)status); }\
 fprintf(stderr,"\n%s",MyMes);				       \
return(this->m_Status);} 

#define CD_Warning							\
fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)this->m_Status, __FILE__, __LINE__); 

#define CD_WarningStatus(status){this->m_Status = status; \
    if (this->m_Status > 0) {						\
      fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
	      (int)this->m_Status, __FILE__, __LINE__); }		\
  }

#define CD_WarningStatusMes(status,MyMes) {this->m_Status = status;\
fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)this->m_Status, __FILE__, __LINE__); \
 printStatusMes((int)this->m_Status);	   \
fprintf(stderr,"\n%s",MyMes); }

#define CD_WarningMes(MyMes){\
fprintf(stderr,"\nWARNING !!! Anomalous status %d [File:'%s' Line=%d]", \
(int)this->m_Status, __FILE__, __LINE__); \
fprintf(stderr,"\n%s",MyMes); \
printStatusMes((int)this->m_Status);	   \
}
#define CD_returnOK {this->m_Status = 0;return(this->m_Status);}

#endif
