/*
 * Project: Spherelib
 * Date: 08/10  
 * Authors: J.M. Colley - M. Le Jeune
 *
 */

/* Copyright (C) 2008 APC CNRS Université Paris Diderot <lejeune@apc.univ-paris7.fr>  
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.html
 */

#include "CD_CommonDefinition.h"
#include <string>
#include "healpix_map.h"
#include <gsl/gsl_blas.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_linalg.h>
#include "AlmExt.h"
#include "MapExt.h"
#include "PowSpecExt.h"
#include "CovMat.h"


using namespace std;

/******************************************************************************
 * 
 * Print pixel value between two indexes
 *
 */
template <typename T> 
void CovMat<T>::PrintCovMat(long begin, long end, const string &title) 
{
  cout << "\n"<<title; //print title of the map
  char Mes[500];
  // check the index consistency
  if (this->m_NbBin==0) {    CD_WarningStatus(1);    return;  }
  
  if (begin >= this->m_NbBin) {
    CD_WarningStatus(8);
    begin = this->m_NbBin;
  }    
  if (end >= this->m_NbBin) {
    CD_WarningStatus(8);
    end = this->m_NbBin;
  }
  if (begin < 0) {CD_WarningStatus(9);    return;  }    
  if (end < 0)   {CD_WarningStatus(9);    return;  }
  
  for(long idx=begin; idx <=  end; idx++) {
    for (long nc=0; nc < m_NbCross; nc++)
      cout << this->m_CovMat[nc][idx] << " "; //print value 
    cout << "\n";
  }  
  cout << "\n";
}
template void CovMat<double>::PrintCovMat(long begin, long end, const string &title) ;
template void CovMat<float>::PrintCovMat(long begin, long end, const string &title) ;

