/*
 * Project: Spherelib
 * Date: 08/10  
 * Authors: J.M. Colley - M. Le Jeune
 *
 */

/* Copyright (C) 2008 APC CNRS Université Paris Diderot <lejeune@apc.univ-paris7.fr>  
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.html
 */

#define HME_DBG


#include "healpix_base.h"
#include "healpix_map.h"
#include "CD_CommonDefinition.h"
#include "MapExt.h"

using namespace std;

/******************************************************************************
 * 
 * Print pixel value between two indexes
 *
 */
template <typename T> 
void MapExt<T>::PrintMap(long begin, long end, const string &title) 
{
  cout << "\n"<<title; //print title of the map
  char Mes[500];
  // check the index consistency
  if (this->npix_==0) {    CD_WarningStatus(1);    return;  }

  if (begin >= this->npix_) {
    sprintf(Mes,"\nNpix = %d - %d\n", begin,this->npix_); 
    CD_WarningStatusMes(8, Mes);
    begin = this->npix_;
  }    
  if (end >= this->npix_) {
    sprintf(Mes,"\nNpix = %d - %d\n", end,this->npix_); 
    CD_WarningStatusMes(8, Mes);
    end = this->npix_;
  }
  if (begin < 0) {CD_WarningStatus(9);    return;  }    
  if (end < 0)   {CD_WarningStatus(9);    return;  }
  
  for(long idx=begin; idx <=  end; idx++) {
    cout << (*this)[idx] << " "; //print pixel values of the map
  }  
  cout << "\n";
}
template void MapExt<double>::PrintMap(long begin, long end, const string &title) ;
template void MapExt<float>::PrintMap(long begin, long end, const string &title) ;
template void MapExt<char>::PrintMap(long begin, long end, const string &title) ;
template void MapExt<int>::PrintMap(long begin, long end, const string &title) ;

/******************************************************************************
 *
 * Print attribut of class Healpix_Map
 */
template <typename T> 
void MapExt<T>::PrintAttribut(const string &Name) 
{
  static string S_StringScheme[]={"RING", "NESTED"};
  
  cout << "\nMapExt object Name :'" << Name << "'";
  cout << "\n  Nside   : " << this->nside_;
  cout << "  Nb pixel  : " << this->npix_;
  if ((this->scheme_ == 0) || (this->scheme_ == 1)) {
    cout << "\n  Scheme  : " << S_StringScheme[this->scheme_];
  }
  else {
    cout << "\n  Scheme  : Error value " << this->scheme_;
  }
  cout << "\n  npface  : " << this->npface_;
  cout << "\n  ncap    : " << this->ncap_;
  cout << "\n  fact1   : " << this-> fact1_;   
  cout << "  fact2: " << this-> fact2_;   
  cout << "\n  Status  : " << m_Status; 
  //  cout << "\n  & arr   : " << this->Map(); 
  PrintMap(0, 11, "  First pixel values: ");
}
template void MapExt<double>::PrintAttribut(const string &Name);
template void MapExt<float>::PrintAttribut(const string &Name); 
template void MapExt<char>::PrintAttribut(const string &Name); 
template void MapExt<int>::PrintAttribut(const string &Name); 

