/*
 * Project: Spherelib
 * Date: 08/10  
 * Authors: J.M. Colley - M. Le Jeune
 *
 */

/* Copyright (C) 2008 APC CNRS Université Paris Diderot <lejeune@apc.univ-paris7.fr>  
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.html
 */

#ifndef POWSPECEXT_H
#define POWSPECEXT_H

#include <stdlib.h>
#include <string.h>
#include "alm_healpix_tools.h"
#include "CD_CommonDefinition.h"
#include "powspec.h"
#include "xcomplex.h"
#include "alm.h" 
#include "healpix_base.h"
#include "fitshandle.h"
//#include "alm_map_tools.h"
#include "healpix_data_io.h"
#include "powspec.h"
#include <stdio.h>

using namespace std; 

template <typename T> class CovMat;
template <typename T> class AlmExt;
template <typename T> class MapExt;


/////////////////////////////////////////////////
/// PowSpecExt is an extension of the Healpix class PowSpec
/////////////////////////////////////////////////
template<typename T> 
class PowSpecExt : public PowSpec
{
 public:
    // Constructor
    PowSpecExt() : PowSpec() {InitAttribut();}
    PowSpecExt(int nspec, int lmax) : PowSpec(nspec, lmax) {InitAttribut();}

    // Destructor
    virtual ~PowSpecExt() {}

    
// ====
// FUNC
// ====

// Getter 
    long GetStatus() {return(m_Status);}


// Pretty print
    // print pixel value between begin and end 
    virtual Status PrintPowSpec(int lbegin, int lend, string title="");
    virtual Status PrintPowSpec(string title="") 
        {CD_return(PrintPowSpec(0, this->Lmax(), title));}
    
    // Print attribut of class
    virtual void PrintAttribut(string Name="");

   
    // Compute power spectrum  and cross power spectrum
    // Alm Object
    Status FromAlm(const AlmExt<T> &alm)
    {
      extract_powspec(alm, *this); 
      CD_return(0);
    }
    Status FromAlm(const AlmExt<T> &alm1, const AlmExt<T> &alm2)
    {
      extract_crosspowspec(alm1, alm2, *this); 
      CD_return(0);
    }

    Status FromPolarAlm(const AlmExt<T> &almT,const AlmExt<T> &almE, const AlmExt<T> &almB)
    {
      extract_powspec(almT,almE, almB, *this); 
      CD_return(0);
    }

    // Map Object
    Status FromMap(MapExt<T> &map)
    {
      AlmExt<double>  Alm;
      Alm.Set(this->Lmax(), this->Lmax());
      CD_iCheck( Alm.FromMap(map) ) ;
      CD_iCheck( FromAlm(Alm) );
      CD_return(0);
    }
    
    Status FromMap(MapExt<T> &map1,MapExt<T> &map2)
    {
      AlmExt<double>  Alm1;
      AlmExt<double>  Alm2;
      
      Alm1.Set(this->Lmax(), this->Lmax());
      Alm2.Set(this->Lmax(), this->Lmax());  
      CD_iCheck( Alm1.FromMap(map1) );   
      CD_iCheck( Alm2.FromMap(map2) );
      CD_iCheck( FromAlm(Alm1, Alm2) );
      CD_return(0);
    }
                  
protected :
// ====
// FUNC
// ====
    void InitAttribut() {
       m_Status=0; 
      }

// ====
// DATA
// ====
    long m_Status;   
};


// 



//
// Template class => define methods directely in header
// 

//#include "PowSpecExt.cpp"
#endif
