// Copyright (C) 2008 Marc Betoule

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// report bugs at betoule@apc.univ-paris7.fr

#include "processoptions.h"
#include <iostream>
#include <sstream>
#include <cstdlib>
Args::Args(int noptions, string * longoptions, char * shortoptions, ArgType * type, string * usage, string _purpose, string _fullusage){
  longOpts = new struct option [noptions+1];
  shortOpts = "";
  optionroles = "";
  for (int i = 0; i < noptions; i++){
    types[shortoptions[i]] = type[i];
    longnames[shortoptions[i]] = longoptions[i];
    shortnames[longoptions[i]] = shortoptions[i];
    isset[longoptions[i]] = false;
    switch (type[i]){
    case STRING :
      {
	stringvalues[longoptions[i]] = "";
	struct option o = {longoptions[i].c_str(), required_argument, NULL, shortoptions[i]};
	longOpts[i] = o;
	shortOpts += shortoptions[i];
	shortOpts += ":";
	optionroles += "\n--"+longoptions[i]+", -"+shortoptions[i]+ " <string>\t: " + usage[i];  
      }
      break;
    case FLOAT :
      {
	floatvalues[longoptions[i]] = 0.0;
	struct option o = {longoptions[i].c_str(), required_argument, NULL, shortoptions[i]}; 
	longOpts[i] = o;
	shortOpts += shortoptions[i];
	shortOpts += ":";
	optionroles += "\n--"+longoptions[i]+", -"+shortoptions[i]+ " <float>\t: " + usage[i];
      }
      break;
    case FLAG :
      {
	flagvalues[longoptions[i]] = 0;
	struct option o = {longoptions[i].c_str(), no_argument, NULL, shortoptions[i]};
	longOpts[i] = o;
	shortOpts += shortoptions[i];
	optionroles += "\n--"+longoptions[i]+", -"+shortoptions[i]+ "\t: " + usage[i];
      }
      break;
    case INT :
      {
	intvalues[longoptions[i]] = 0;
	struct option o = {longoptions[i].c_str(), required_argument, NULL, shortoptions[i]};
	longOpts[i] = o;
	shortOpts += shortoptions[i];
	shortOpts += ":";
	optionroles += "\n--"+longoptions[i]+", -"+shortoptions[i]+ " <int>\t: " + usage[i];
      }
      break;
    }
    purpose = _purpose;
    fullusage = _fullusage;
  }
  struct option o = {"help", no_argument, NULL, 'h'};
  longOpts[noptions] = o;
  shortOpts += "h";
  if (flagvalues.size() > 0){
    synopsis += " [-";
    for (map<string, int>::iterator it = flagvalues.begin(); it != flagvalues.end(); it++)
      synopsis += shortnames[it->first];
    synopsis += "] ";
  }
  for (map<string, int>::iterator it = intvalues.begin(); it != intvalues.end(); it++){
    synopsis += "[-";
    synopsis += shortnames[it->first];
    synopsis += " <int>] ";
  }
  for (map<string, double>::iterator it = floatvalues.begin(); it != floatvalues.end(); it++){
    synopsis += "[-";
    synopsis += shortnames[it->first];
    synopsis += " <float>] ";
  }
  
  for (map<string, string>::iterator it = stringvalues.begin(); it != stringvalues.end(); it++){
    synopsis += "[-";
    synopsis += shortnames[it->first];
    synopsis += " <string>] ";
  }
  synopsis += fullusage+"\n";
}

void Args::info(){
  for (map<string, int>::iterator it = intvalues.begin(); it != intvalues.end(); it++)
    cout << it->first << ": " << it->second <<endl;
  for (map<string, double>::iterator it = floatvalues.begin(); it != floatvalues.end(); it++)
    cout << it->first << ": " << it->second <<endl;
  for (map<string, string>::iterator it = stringvalues.begin(); it != stringvalues.end(); it++)
    cout << it->first << ": " << it->second <<endl;
  for (map<string, int>::iterator it = flagvalues.begin(); it != flagvalues.end(); it++)
    cout << it->first << ": " << it->second <<endl;
  cout << nposarg << " positionnal arguments:\n";
  for(int i = 0; i<nposarg; i++)
    cout << posarg[i] << endl;
  
}

void Args::printusage(){
  cerr << synopsis << endl
       << purpose << endl
       << optionroles << endl;
      
  exit(EXIT_FAILURE);
}

void Args::processarg(int val){
  istringstream s;
  if (val == '?'){
    cerr << "Incorrect option: " << (char)optopt << endl;
    printusage();
  }
  if ( val == 'h'){
    cerr << "Usage: " << endl;
    printusage();
  }
  switch (types[val]){
  case STRING:
    stringvalues[longnames[val]] = optarg;
    break;
  case FLOAT:
    s.str(optarg);
    s >> floatvalues[longnames[val]];
    break;
  case FLAG:
    flagvalues[longnames[val]]++;
    break;
  case INT:
    s.str(optarg);
    s >> intvalues[longnames[val]];
    break;
  }
  
  isset[longnames[val]]= true;
}

void Args::processargs(int argc, char **argv){
  int opt = 0;
  int longIndex = 0;
  string name = argv[0];
  synopsis = name.substr(name.find_last_of('/')+1) + synopsis;
  synopsis = "Synopsis:\n" + synopsis;
  opterr = 0;
  opt = getopt_long( argc, argv, shortOpts.c_str(), longOpts, &longIndex );
  while( opt != -1 ) {
    processarg(opt);
    opt = getopt_long( argc, argv, shortOpts.c_str(), longOpts, &longIndex );
  }
  nposarg =argc - optind;
  posarg = new string[nposarg];
  for (int i = optind; i < argc; i++)
    posarg[i - optind] = argv[i];
}

// template <int>
// void Args::get(int prout, string key){
//   //  return intvalues[key];
// }
//double Args::get (string key){return floatvalues[key];}
//string Args::get (string key){return stringvalues[key];}
//bool Args::get (string key){return flagvalues[key];}
bool Args::operator [](string key){
  return isset.find(key)->second;
}

string Args::operator[](int i){
  return posarg[i];
}

void Args::def (int val,string key){
  map<string,int>::iterator it;
  it = intvalues.find(key);
  if (it != intvalues.end() && (!isset[key]))
    it->second = val;
}
void Args::def (double val, string key){
  map<string,double>::iterator it;
  it = floatvalues.find(key);
  if (it != floatvalues.end() && (!isset[key]))
    it->second = val;
}
void Args::def (string  val, string key){
  map<string,string>::iterator it;
  it = stringvalues.find(key);
  if (it != stringvalues.end() && (!isset[key]))
    it->second = val;
}
