// Copyright (C) 2008 Marc Betoule

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// report bugs at betoule@apc.univ-paris7.fr


#include <getopt.h>
#include <string>
#include <map>

using namespace std;

enum ArgType {STRING, FLAG, INT, FLOAT};

class Args
{
public:
  // constructor
  Args(int noptions, string * longoptions, char * shortoptions, ArgType * type, string * usage, string purpose="", string fullusage="");
  
  // various
  void info();
  void printusage();
  
  // process
  void processargs(int argc, char **argv);

  // options accessors
  void get (int & val,string key){
    map<string,int>::iterator it;
    it = intvalues.find(key);
    if (it != intvalues.end())
      val = it->second;
  }
  void get (double & val, string key){
    map<string,double>::iterator it;
    it = floatvalues.find(key);
    if (it != floatvalues.end())
      val = it->second;
  }
  void get (string & val, string key){
    map<string,string>::iterator it;
    it = stringvalues.find(key);
    if (it != stringvalues.end())
      val = it->second;
  }
  
  bool operator [](string key);
  string operator [](int pos);

  // options modifier
  void def (int val,string key);
  void def (double val, string key);
  void def (string  val, string key);
private:
  
  // various info
  string purpose;
  string fullusage;
  string shortOpts;
  string optionroles;
  string synopsis;
  struct option * longOpts;
  
  // options storage
  
  map<string,int> intvalues;
  map<string,string> stringvalues;
  map<string,double> floatvalues;
  map<string,int> flagvalues;
  string * posarg;
  int nposarg;

  map<int,ArgType> types;
  map<int,string> longnames;
  map<string,int> shortnames;
  map<string, bool> isset;
  // process
  void processarg(int val);
  
};


